/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

class CompetitionController {

  def submissionService

  def scaffold = Competition

  def ready = {
  }

  def sendReady = {
    String compIdString = params.serverId
    Long compId
    if (!compIdString.isEmpty()) {
      compId = compIdString?.toLong()
    }
    submissionService.submitReadyNotification(compId)
    flash.message="Ready notification sent to ${grailsApplication.config.tacenergy.server}."
    redirect(uri: '/')
  }

  def deleteAll = {
    Competition.list()?.each {competitionInstance ->
      deleteAllData()
    }
    flash.message = "All data deleted."
    redirect(uri: "/")
  }

  def delete = {
    def competitionInstance = Competition.get(params.id)
    if (competitionInstance) {
      try {
        deleteCompetionDependencies(competitionInstance)
        competitionInstance.delete ()
        flash.message = "${message(code: 'default.deleted.message', args: [message(code: 'competition.label', default: 'Competition'), params.id])}"
        redirect(action: "list")
      }
      catch (Exception e) {
        flash.message = "${message(code: 'default.not.deleted.message', args: [message(code: 'competition.label', default: 'Competition'), params.id])}"
        log.error "Competition cannot be deleted: ${e.message} (${e.cause})"
        redirect(action: "show", id: params.id)
      }
    }
    else {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'competition.label', default: 'Competition'), params.id])}"
      redirect(action: "list")
    }
  }


  private void deleteCompetionDependencies(Competition competitionInstance) {
    Forecast.executeUpdate('delete from Forecast f where f.competition = ?', [competitionInstance])
    DepotPosition.executeUpdate('delete from DepotPosition d where d.competition = ?', [competitionInstance])
    CashPosition.executeUpdate('delete from CashPosition c where c.competition = ?', [competitionInstance])
    Orderbook.executeUpdate('delete from Orderbook o where o.competition = ?', [competitionInstance])
    Shout.executeUpdate('delete from Shout s where s.competition = ?', [competitionInstance])
    TradeLog.executeUpdate('delete from TradeLog t where t.competition = ?', [competitionInstance])
    QuoteLog.executeUpdate('delete from QuoteLog q where q.competition = ?', [competitionInstance])
    Product.executeUpdate('delete from Product p where p.competition = ?', [competitionInstance])
  }

  private void deleteAllData() {
    Forecast.executeUpdate('delete from Forecast')
    DepotPosition.executeUpdate('delete from DepotPosition')
    CashPosition.executeUpdate('delete from CashPosition')
    Orderbook.executeUpdate('delete from Orderbook')
    Shout.executeUpdate('delete from Shout')
    TradeLog.executeUpdate('delete from TradeLog')
    QuoteLog.executeUpdate('delete from QuoteLog')
    Product.executeUpdate('delete from Product')
    Competition.executeUpdate('delete from Competition')
  }
}
