/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

class ShoutController {


  def scaffold = Shout

  def submissionService

  def save = {
    def shoutInstance = new Shout(params)
    if (shoutInstance.validate()) {
      submissionService.submitShout(shoutInstance)
      flash.message = "Shout was submitted to the server. Please wait and refresh the view from time to time to see the incoming order notification from the server."
      shoutInstance.discard()
      redirect(action: "list")
    }
    else {
      render(view: "create", model: [shoutInstance: shoutInstance])
    }
  }

  def update = {
    def shoutInstance = Shout.get(params.id)
    if (shoutInstance) {
      if (params.version) {
        def version = params.version.toLong()
        if (shoutInstance.version > version) {

          shoutInstance.errors.rejectValue("version", "default.optimistic.locking.failure", [message(code: 'shout.label', default: 'Shout')] as Object[], "Another user has updated this Shout while you were editing")
          render(view: "edit", model: [shoutInstance: shoutInstance])
          return
        }
      }
      shoutInstance.properties = params
      if (!shoutInstance.hasErrors() && shoutInstance.save(flush: true)) {
        flash.message = "${message(code: 'default.updated.message', args: [message(code: 'shout.label', default: 'Shout'), shoutInstance.id])}"
        redirect(action: "show", id: shoutInstance.id)
      }
      else {
        render(view: "edit", model: [shoutInstance: shoutInstance])
      }
    }
    else {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
      redirect(action: "list")
    }
  }

  def delete = {
    def shoutInstance = Shout.get(params.id)
    if (shoutInstance) {
      try {
        shoutInstance.delete(flush: true)
        flash.message = "${message(code: 'default.deleted.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
        redirect(action: "list")
      }
      catch (org.springframework.dao.DataIntegrityViolationException e) {
        flash.message = "${message(code: 'default.not.deleted.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
        redirect(action: "show", id: params.id)
      }
    }
    else {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
      redirect(action: "list")
    }
  }
}
