/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

class Competition {

  Long serverId
  String name
  CompetitionStatus competitionStatus
  String description
  Boolean current
  Date currentCompetitionTime
  Long initialDuration //seconds before and initial deactivation / activation cylce is started
  Long durationBetweenShifts //seconds between deactivation of oldest and activation of newest product
  Integer timeslotLength //minutes
  Integer timeslotsOverall //overall timeslotsOverall
  Integer timeslotsOpen //concurrently open timeslotsOverall (time window)
  Integer deactivateTimeslotsAhead
  Date simulationStartTime
  Integer participantCount
  Double balancingCostOver
  Double balancingCostUnder
  ActionType action

  static constraints = {
    serverId(nullable: true)
    name(unique: false, blank: false)
    competitionStatus(nullable: false)
    description(nullable: true)
    current(nullable: true)
    currentCompetitionTime (nullable: true)
    initialDuration(nullable: false, min: 1l)
    durationBetweenShifts(nullable: false, min: 1l)
    timeslotLength(nullable: false, min: 1)
    timeslotsOverall(nullable: false, min: 1)
    timeslotsOpen(nullable: false, min: 1, validator: { timeslotsOpen, competition ->
      timeslotsOpen <= (competition.timeslotsOverall - competition.deactivateTimeslotsAhead)
    })
    deactivateTimeslotsAhead(nullable: false, min: 0, validator: {deactivateTimeslotsAhead, competition ->
      deactivateTimeslotsAhead <= (competition.timeslotsOverall - competition.timeslotsOpen)
    })
    simulationStartTime(nullable: false)
    participantCount(nullable: false, min: 0, max: 20)
    balancingCostOver(nullable: false)
    balancingCostUnder(nullable: false)
    action(nullable: true)
  }

  public String toString() {
    return name
  }

  public void copyPropertiesFrom(Competition other) {
    this.serverId = other.serverId
    this.name = other.name
    this.competitionStatus = other.competitionStatus
    this.description = other.description
    this.current = other.current
    this.currentCompetitionTime = other.currentCompetitionTime
    this.initialDuration = other.initialDuration
    this.durationBetweenShifts = other.durationBetweenShifts
    this.timeslotLength = other.timeslotLength
    this.timeslotsOverall = other.timeslotsOverall
    this.timeslotsOpen = other.timeslotsOpen
    this.deactivateTimeslotsAhead = other.deactivateTimeslotsAhead
    this.simulationStartTime = other.simulationStartTime
    this.participantCount = other.participantCount
    this.balancingCostOver = other.balancingCostOver
    this.balancingCostUnder = other.balancingCostUnder
    this.action = other.action
  }
}
