/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

class Product {

  Long serverId
  Competition competition
  String name
  Boolean enabled = false
  Date startDateTime
  Date endDateTime
  Long serialNumber
  Boolean current
  ActionType action

  SortedSet orderbooks
  static hasMany = [depotPosition: DepotPosition, shouts: Shout, orderbooks: Orderbook]

  static constraints = {
    serverId(nullable: true)
    competition(nullable: false)
    name(nullable: false, blank: false)
    startDateTime(nullable: false)
    endDateTime(nullable: false)
    current(nullable: false)
    serialNumber(nullable: false)
    action(nullable: true)
  }

  public String toString() {
    return name;
  }

  public Product next() {
    if (this.competition) {
      return Product.findByCompetitionAndSerialNumber(this.competition, this.serialNumber + 1l)
    } else {
      return Product.findByCompetitionIsNullAndSerialNumber(this.serialNumber + 1l)
    }
  }

  public Product previous() {
    if (this.competition) {
      return Product.findByCompetitionAndSerialNumber(this.competition, this.serialNumber - 1l)
    } else {
      return Product.findByCompetitionIsNullAndSerialNumber(this.serialNumber - 1l)
    }
  }

  public void copyPropertiesFrom(Product other) {
    this.serverId = other.serverId
    this.competition = other.competition
    this.name = other.name
    this.enabled = other.enabled
    this.startDateTime = other.startDateTime
    this.endDateTime = other.endDateTime
    this.serialNumber = other.serialNumber
    this.current = other.current
    this.action = other.action
  }
}