/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

import grails.converters.XML
import org.codehaus.groovy.grails.commons.ConfigurationHolder

class Shout implements Serializable {

  Long serverId
  Competition competition
  Product product
  BuySellIndicator buySellIndicator
  BigDecimal remainingQuantity
  BigDecimal limitPrice
  BigDecimal executionQuantity
  BigDecimal executionPrice
  OrderType orderType = OrderType.MARKET
  Date dateCreated = new Date()
  Date dateMod = new Date()
  ModReasonCode modReasonCode = ModReasonCode.INSERT
  Boolean outdated = false
  Long transactionID
  Long shoutID
  AuctionTradeFlag auctionTradeFlag

  static belongsTo = [product: Product]

  static constraints = {
    serverId(nullable: true)
    competition(nullable: true)
    product(nullable: false)
    buySellIndicator(nullable: false)
    remainingQuantity(nullable: false, min: 0.0, scale: 2)
    limitPrice(nullable: true, min: 0.0, scale: 2, validator: { limitPrice, sh1 ->
      (
      (sh1.orderType == OrderType.MARKET && limitPrice == null)
          || (sh1.orderType == OrderType.LIMIT && limitPrice != null)
      )
    })
    executionQuantity(nullable: true, min: 0.0, scale: 2)
    executionPrice(nullable: true, min: 0.0, scale: 2)
    orderType(nullable: false)
    dateCreated(nullable: true)
    dateMod(nullable: true)
    modReasonCode(nullable: true)
    outdated(nullable: true)
    transactionID(nullable: true, min: 0L)
    auctionTradeFlag(nullable: true)
    shoutID(nullable: true, min: 0L)
  }


   public void copyPropertiesFrom(Shout other) {
    this.serverId = other.serverId
    this.competition = other.competition
    this.product = other.product
    this.buySellIndicator = other.buySellIndicator
    this.remainingQuantity = other.remainingQuantity
    this.limitPrice = other.limitPrice
    this.executionQuantity = other.executionQuantity
    this.executionPrice = other.executionPrice
    this.orderType = other.orderType
    this.dateCreated = other.dateCreated
    this.dateMod = other.dateMod
    this.modReasonCode = other.modReasonCode
    this.outdated = other.outdated
    this.transactionID = other.transactionID
    this.auctionTradeFlag = other.auctionTradeFlag
    this.shoutID = other.shoutID
  }

  public void deleteCompleteOrder() {
    def allOrderParts = Shout.findAllByShoutID(this.shoutID)
    for (Shout orderPart: allOrderParts) {
      orderPart.delete()
    }
  }

  public ShoutSubmitCommand toShoutSubmitCommand() {
    def shoutSubmitCommand = new ShoutSubmitCommand()
    shoutSubmitCommand.productId = this.product?.serverId
    shoutSubmitCommand.username = ConfigurationHolder.config?.tacenergy?.username
    shoutSubmitCommand.apikey = ConfigurationHolder.config?.tacenergy?.apikey
    shoutSubmitCommand.buySellIndicator = this.buySellIndicator
    shoutSubmitCommand.remainingQuantity = this.remainingQuantity
    shoutSubmitCommand.limitPrice = this.limitPrice
    shoutSubmitCommand.orderType = this.orderType
    return shoutSubmitCommand
  }

  public String toShoutSubmitCommandXml() {
    return toShoutSubmitCommand() as XML
  }

  public ShoutUpdateCommand toShoutUpdateCommand() {
    def shoutUpdateCommand = new ShoutUpdateCommand()
    shoutUpdateCommand.username = ConfigurationHolder.config?.tacenergy?.username
    shoutUpdateCommand.apikey = ConfigurationHolder.config?.tacenergy?.apikey
    shoutUpdateCommand.shoutID = this.shoutID
    shoutUpdateCommand.remainingQuantity = this.remainingQuantity
    shoutUpdateCommand.limitPrice = this.limitPrice
    shoutUpdateCommand.orderType = this.orderType
    return shoutUpdateCommand
  }

  public String toShoutUpdateCommandXml() {
    return toShoutUpdateCommand() as XML
  }

  public ShoutDeleteCommand toShoutDeleteCommand() {
    def shoutDeleteCommand = new ShoutDeleteCommand()
    shoutDeleteCommand.username = ConfigurationHolder.config?.tacenergy?.username
    shoutDeleteCommand.apikey = ConfigurationHolder.config?.tacenergy?.apikey
    shoutDeleteCommand.shoutID = this.shoutID
    return shoutDeleteCommand
  }

  public String toShoutDeleteCommandXml() {
    return toShoutDeleteCommand() as XML
  }
}
