/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

import grails.converters.XML

class SubmissionService {

  boolean transactional = true
  def jmsTemplate

  def submitShout(ShoutSubmitCommand shout) {
    if (!shout) throw new NullPointerException("submitShout triggered with null object as shout")
    String shoutXml = shout as XML
    sendShoutXml(shoutXml)
  }

  def submitShout(Shout shout) throws NullPointerException {
    if (!shout) throw new NullPointerException("submitShout triggered with null object as shout")
    sendShoutXml(shout.toShoutSubmitCommandXml())
  }

  def updateShout (ShoutUpdateCommand shout) throws NullPointerException {
    if (!shout) throw new NullPointerException("updateShout triggered with null object as shout")
    String shoutXml = shout as XML
    sendShoutXml(shoutXml)
  }

  def updateShout(Shout shout) throws NullPointerException {
    if (!shout) throw new NullPointerException("updateShout triggered with null object as shout")
    sendShoutXml(shout.toShoutUpdateCommandXml())
  }

  def deleteShout(ShoutDeleteCommand shout) throws NullPointerException {
    if (!shout) throw new NullPointerException("deleteShout triggered with null object as shout")
    String shoutXml = shout as XML
    sendShoutXml(shoutXml)
  }

  def deleteShout(Shout shout) throws NullPointerException {
    if (!shout) throw new NullPointerException("deleteShout triggered with null object as shout")
    sendShoutXml(shout.toShoutDeleteCommandXml())
  }

  private void sendShoutXml(String shoutXml) {
    jmsTemplate.convertAndSend("public.ShoutInputQueue", shoutXml)
    log.debug "Shout submitted to market: ${shoutXml}"
  }


  def submitReadyNotification(Long compId) {
    def readyNotificationCommand = new ReadyNotificationCommand()
    readyNotificationCommand.competitionId = compId
    String readyNotificationXml = readyNotificationCommand as XML
    try {
      jmsTemplate.convertAndSend("public.ClientAnnounce", readyNotificationXml)
      log.debug "Ready Notification submitted to market: ${readyNotificationXml}"
    } catch (Exception e) {
      log.error("Failed to send ready notification to market: ${readyNotificationXml}", e)
    }
  }
}
