/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.tacdemo

import groovy.util.slurpersupport.GPathResult
import org.springframework.beans.BeanWrapper
import org.springframework.beans.BeanWrapperImpl
import org.codehaus.groovy.grails.commons.ConfigurationHolder

class XmlDataBinderService {
  boolean transactional = true
  def grailsApplication
  def config = ConfigurationHolder.config

  public Object bindXmlData(Class targetClass, String source, List properties) {
    if (targetClass == null || source == null || properties == null) {
      log.warn "bindXmlDate called with null references: targetClass: ${targetClass}, source: ${source}, properties: ${properties?.toListString()}"
      return null
    }
    def gpathSource = new XmlSlurper().parseText(source)
    return bindXmlData(targetClass, gpathSource, properties)
  }

  public Object bindXmlData(Class targetClass, GPathResult source, List properties) {
    if (targetClass == null || source == null || properties == null) return null
    def targetObject = grailsApplication.classLoader.loadClass(targetClass.name).newInstance()
    if (targetObject) {
      return bindXmlData(targetObject, source, properties)
    } else {
      return null
    }
  }

  public Object bindXmlData(Object target, String source, List properties) {
    if (target == null || source == null || properties == null) return null
    def gpathSource = new XmlSlurper().parseText(source)
    return bindXmlData(target, gpathSource, properties)
  }

  public Object bindXmlData(Object target, GPathResult source, List properties) {
    if (target == null || source == null || properties == null) return null
    BeanWrapper wrapper = new BeanWrapperImpl()
    try {
      wrapper.registerCustomEditor(Date.class, new CustomDateBinder())
      wrapper.setWrappedInstance(target)
      properties.each {property ->
        if (property instanceof Map) {//This indicates a domain class to bind e.g. experiment.id -> Experiment
          property.each {String targetProperty, domainClass ->
            def id = source[targetProperty]?.toString()
            if (id != null) {
              def subdomainInstance = null
              try {
                log.debug "Found key ${targetProperty}, value: ${domainClass}: Trying to create related domain object based on serverId ${id}"
                int i=1
                while (!subdomainInstance && i<=config.tacenergy.databinding.retryCount) {
                  log.debug "Try $i for finding ${id} in domain ${domainClass}"
                  subdomainInstance = domainClass.findByServerId(id)
                  log.debug ("Instance ${subdomainInstance} found.")
                  if (!subdomainInstance) {
                    Thread.currentThread().sleep(config.tacenergy.databinding.retryWait)

                  }
                  i++
                }
              } catch (Exception ex) {
                log.error "Failed to bind object $targetProperty with id ${id} to object ${target}", ex
              }
              if (subdomainInstance != null) {
                def propertyName = targetProperty.substring(0,targetProperty.length()-2) //take e.g. "competition" part out of "competitionId"
                log.debug "Trying to set property ${propertyName} to ${subdomainInstance}"
                wrapper.setPropertyValue(propertyName, subdomainInstance)
              }
            }
          }
        } else if (property.equals('id')) {
          //ignore id property as this is set automatically by the database
        } else if (property.equals('version')) {
          //ignore version property as this is set automatically by the database
        } else { //regular attributes
          def prop = source[property]?.toString()
          if (prop != null) {
            log.debug "Trying to set property ${property} to ${prop}"
            wrapper.setPropertyValue(property, prop)
          }
        }
      }
    } catch (Exception e) {
      log.error ("An error occurred while binding xml data ${source} to target object ${target} using target properties ${properties}",e)
      return null
    }
    return target
  }
}
